local game = {}

function game:load()
	self.text = {
		pc = {
			start = "click to start\n("..config.controls.jump..")",
			pause = "click to pause\n("..config.controls.pause..")",
			resume = "click to resume\n("..config.controls.jump..")",
			reset = "click to restart\n("..config.controls.jump..")",
			info = "Press '"..config.controls.jump.."' or click to jump"
		},
		mobile = {
			start = "tap to start",
			pause = "tap to pause",
			resume = "tap to resume",
			reset = "tap to reset",
			info = "tap to jump"
		}
	}

	self.title = {
		y = -(config.display.height * 0.5),
		targetY = config.display.height * 0.2,
		offY = -(config.display.height * 0.5)
	}

	self.hud = {
		x = -(config.display.width * 0.5),
		targetX = -(config.display.width * 0.5),
		offX = -(config.display.width * 0.5)
	}

	self.canvas = {
		entity = love.graphics.newCanvas(config.display.width, config.display.height),
		gui = love.graphics.newCanvas(config.display.width, config.display.height),
		shader = love.graphics.newCanvas(config.display.width, config.display.height)
	}

	self.first = true

	self:reset()
end

function game:reset()
	physics:load()
	--Creating World
	self.ground = {
		x = 0,
		y = math.floor(config.display.height * 0.8),
		width = config.display.width * 2,
		height = config.display.height - math.floor(config.display.height * 0.8),
		color = {227, 210, 150, 255},
		type = "GROUND"
	}
	self.sky = verticalGradient(config.display.width, config.display.height - (config.display.height - self.ground.y), {117, 172, 198}, {178, 220, 243})

	--GAME VARIABLES
	--State
	self.started = false
	self.paused = false
	self.ended = false
	self.newHighScore = false
	self.distance = 0
	self.lives = 0

	self.inputDelay = 1
	self.inputDelayTick = 0
	self.takeInput = true

	--settings
	self.gameSpeed = 1
	self.gameSpeedTick = 0
	self.gameSpeedTime = 10
	self.time = 0

	--Obstacles
	self.obstacleSpawnRate = 0.5
	self.obstacleSpawnTick = 0
	self.obstacleSpeed = config.display.width * 0.5

	--Creating Entities
	entity:clear()

	physics:add(self.ground)
	self.player = entity:spawn("player", {ground = self.ground.y, gameSpeed = self.gameSpeed}, "player")

	local credits = [[
CREDITS:
'bump' library by Enrique García Cota
(kikito)

'Pixel operator' font by Jayvee Enaguas
(HarvettFox96)

Made in löve]]
	self.textObject = {
		start = text.new(self.text[platform].start, 0, config.display.height * 0.85, {32, 32, 32, 255}, font.small, "center", true, self.textClick, "start"),
		pause = text.new(self.text[platform].pause, 0, config.display.height * 0.85, {32, 32, 32, 255}, font.small, "center", true, self.textClick, "pause"),
		resume = text.new(self.text[platform].resume, 0, config.display.height * 0.85, {32, 32, 32, 255}, font.small, "center", true, self.textClick, "resume"),
		reset = text.new(self.text[platform].reset, 0, config.display.height * 0.85, {32, 32, 32, 255}, font.small, "center", true, self.textClick, "reset"),
		distance = text.new(0, config.display.width * 0.08, config.display.height * 0.02, {240, 240, 240, 255}, font.large, "left"),
		stats = text.new("top distance: "..config.game.topDistance.."\ntotal distance: "..config.game.totalDistance, config.display.width * 0.01, config.display.height * 0.02, {240, 240, 240, 255}, font.small, "left"),
		highScore = text.new("new high score!", 0, config.display.height * 0.3, {247, 205, 37, 255}, font.small, "center"),
		gameOver = text.new("You lost!", 0, config.display.height * 0.1, {222, 76, 76, 255}, font.large, "center"),
		info = text.new(self.text[platform].info, 0, config.display.height * 0.7, {240, 240, 240, 255}, font.tiny, "center"),
		credits = text.new("credits", -(config.display.width * 0.01), config.display.height * 0.92, {32, 32, 32, 255}, font.small, "right", true, self.textClick, "credits"),
		back = text.new("back", 0, config.display.height * 0.85, {32, 32, 32, 255}, font.small, "center", true, self.textClick, "back"),
		creditsString = text.new(credits, config.display.width * 0.01, config.display.height * 0.05, {240, 240, 240, 255}, font.small, "left"),
		soundButton = button.new(soundButton1, config.display.width * 0.01, config.display.height * 0.83, self.textClick, "sound"),
		loveButton = button.new(loveLogo, config.display.width * 0.01, config.display.height * 0.55, self.textClick, "love")
	}

	self.textObject.start:show()
	self.textObject.stats:show()
	self.textObject.info:show()
	self.textObject.pause:hide()
	self.textObject.resume:hide()
	self.textObject.reset:hide()
	self.textObject.distance:hide()
	self.textObject.highScore:hide()
	self.textObject.gameOver:hide()

	self.textObject.credits:show()
	self.textObject.back:hide()
	self.textObject.back:hide()
	self.textObject.creditsString:hide()

	self.textObject.soundButton:show()

	self.title.targetY = config.display.height * 0.2

	self.screenShake = {
		tick = 0,
		duration = 0.1,
		magnitude = config.display.width * 0.005
	}

	self.trip = false
	self.tripDuration = 8
	self.tripMagnitude = 1
	self.tripTick = 0
	self.tripRising = false
	self.tripRiseSpeed = 1

	if not self.first then
		sound:play("ui1")
	end
	if self.first then
		self.first = false
	end

	wave:send("strength", 1)
	self:setVolume()
	sound:stopAll()
end

function game:start()
	self.started = true
	self.player:run()

	self.textObject.start:hide()
	self.textObject.credits:hide()
	self.textObject.stats:hide()
	self.textObject.info:hide()
	self.textObject.pause:show()
	self.textObject.distance:show()
	self.textObject.soundButton:hide()
	self.title.targetY = self.title.offY
	self.hud.targetX = config.display.width * 0.01
	sound:play("ui1")
	sound:play("run")
end

function game:shake()
	self.screenShake.tick = self.screenShake.duration
end

function game:lose()
	sound:stopAll()
	game:stopTrip()

	--Input Delay
	self.takeInput = false
	self.inputDelayTick = self.inputDelay

	self.ended = true

	self.textObject.gameOver:setText("You lost!\n"..math.floor(self.distance * 100) / 100)

	self.textObject.reset:show()
	self.textObject.gameOver:show()
	self.textObject.pause:hide()
	self.textObject.distance:hide()

	--High score
	if self.distance > config.game.topDistance then
		self.newHighScore = true
		config.game.topDistance = math.floor(self.distance * 100) / 100
		self.textObject.highScore:show()
	end
	config.game.totalDistance = config.game.totalDistance + math.floor(self.distance * 100) / 100

	self.hud.targetX = self.hud.offX

	ini.save("config.ini", config)
	sound:play("hit")
	sound:stop("run")
end

function game:pause()
	if self.started and not self.lost then
		self.paused = true
		self.textObject.pause:hide()
		self.textObject.info:show()
		self.textObject.resume:show()
		self.textObject.soundButton:show()
		sound:play("ui2")
		sound:stop("run")
	end
end

function game:resume()
	self.paused = false
	self.textObject.pause:show()
	self.textObject.info:hide()
	self.textObject.resume:hide()
	self.textObject.soundButton:hide()
	sound:play("ui1")
	sound:play("run")
end

function game:showCredits()
	self.textObject.start:hide()
	self.textObject.info:hide()
	self.textObject.stats:hide()
	self.textObject.credits:hide()
	self.textObject.back:show()
	self.textObject.creditsString:show()
	self.textObject.loveButton:show()
	self.textObject.soundButton:hide()
	self.title.targetY = self.title.offY
end

function game:hideCredits()
	self.textObject.start:show()
	self.textObject.info:show()
	self.textObject.stats:show()
	self.textObject.credits:show()
	self.textObject.back:hide()
	self.textObject.creditsString:hide()
	self.textObject.loveButton:hide()
	self.textObject.soundButton:show()
	self.title.targetY = config.display.height * 0.2
end

function game:addLife()
	self.lives = self.lives + 1
	sound:play("life")
end

function game:startTrip()
	if self.trip then
		self.tripTick = self.tripDuration
	else
		self.trip = true
		self.tripRising = true
		self.tripMagnitude = 0
		self.tripTick = self.tripDuration
	end
	sound:play("trip")
end

function game:stopTrip()
	self.trip = false
	self.tripRising = false
	self.tripMagnitude = 0
	self.tripTick = 0
	sound:stopAll()
end

function game:setVolume(c)
	c = c or false
	if c then
		config.sound.volume = config.sound.volume - 0.25
		if config.sound.volume < 0.25 then
			config.sound.volume = 1
		end
	end

	local realVolume = config.sound.volume
	if config.sound.volume == 0.25 then
		realVolume = 0
	end

	sound:setMasterVolume(realVolume)

	if config.sound.volume == 1 then
		self.textObject.soundButton:setQuad(soundButton1)
	elseif config.sound.volume == 0.75 then
		self.textObject.soundButton:setQuad(soundButton2)
	elseif config.sound.volume == 0.5 then
		self.textObject.soundButton:setQuad(soundButton3)
	elseif config.sound.volume == 0.25 then
		self.textObject.soundButton:setQuad(soundButton4)
	end

	sound:play("ui1")
end

function game:spawnObstacle()
	--Deciding type
	local r = love.math.random()
	local type = "cactus"
	if r < 0.2 then
		r = love.math.random()
		if r > 0.2 then
			type = "mutantCactus"
		elseif r > 0.1 then
			type = "imposterCactus"
		else
			type = "funnyCactus"
		end
	end
	--type = "funnyCactus"
	
	--Spawning
	local c = entity:spawn(type, {ground = self.ground.y, obstacleSpeed = self.obstacleSpeed, gameSpeed = self.gameSpeed})
	physics.add(c)
end

--==[[ UPDATING ]]==--

function game:updateObstacles(dt)
	if self.started and not self.paused and not self.ended then
		self.obstacleSpawnTick = self.obstacleSpawnTick + (dt * self.gameSpeed)
		if self.obstacleSpawnTick > (1 / self.obstacleSpawnRate) then
			game:spawnObstacle()
			--Double Spawn
			local r = math.random()
			if r < 0.4 then
				self.obstacleSpawnTick = (1 / self.obstacleSpawnRate)  / 2
			else
				self.obstacleSpawnTick = 0
			end
		end
	end
end

function game:update(dt)
	if debugMode then
		if love.keyboard.isDown("f4") then
			dt = dt / 4
		end
	end

	if self.trip then
		if self.tripRising then
			self.tripMagnitude = self.tripMagnitude + (self.tripRiseSpeed * dt)
			dt = dt / (self.tripMagnitude + 1)
			wave:send("strength", self.tripMagnitude )
			color:send("strength", self.tripMagnitude )

			if self.tripMagnitude > 1 then
				self.tripMagnitude = 1
				self.tripRising = false
			end
		else
			self.tripTick = self.tripTick - dt
			self.tripMagnitude = fmath.normal(self.tripTick, 0, self.tripDuration)
			dt = dt / (self.tripMagnitude + 1)
			wave:send("strength", self.tripMagnitude )
			color:send("strength", self.tripMagnitude )

			if self.tripTick < 0 then
				self.tripTick = 0
				self.trip = false
			end
		end
	end

	self.time = self.time + dt
	if self.time > math.pi then time = 0 end
	wave:send("time", self.time)
	--color:send("time", self.time)

	if self.screenShake.tick > 0 then
		self.screenShake.tick = self.screenShake.tick - dt
		if self.screenShake.tick < 0 then
			self.screenShake.tick = 0
		end
	end

	self.title.y = self.title.y + math.floor(self.title.targetY - self.title.y) * (6 * dt)
	self.hud.x = self.hud.x + math.floor(self.hud.targetX - self.hud.x) * (6 * dt)

	--Input Delay
	if not self.takeInput then
		self.inputDelayTick = self.inputDelayTick - dt
		if self.inputDelayTick < 0 then
			self.inputDelayTick = 0
			self.takeInput = true
		end
	end

	if self.started and not self.ended and not self.paused then
		local mp = 1
		if self.trip then
			mp = mp * 2
		end
		self.distance = self.distance + (dt * (self.gameSpeed * mp) )

		self.gameSpeedTick = self.gameSpeedTick + dt
		if self.gameSpeedTick > self.gameSpeedTime then
			self.gameSpeed = self.gameSpeed + 0.05
			self.player.gameSpeed = self.gameSpeed
			self.gameSpeedTick = 0
		end

		game:updateObstacles(dt)
		physics:update(dt)
		entity:update(dt)
	end

	popup:update(dt)

	--Updating Text Objects
	self.textObject.distance:setText(math.floor(self.distance * 100) / 100)

	for i,v in pairs(self.textObject) do
		v:update(dt)
	end


	--Sound killer
	if self.ended or self.paused then
		sound:stop("run")
	end
end

--==[[ DRAWING ]]==--

function game:drawUI()
	for i,v in pairs(self.textObject) do
		v:draw()
	end

	popup:draw()

	--UI Images
	love.graphics.setColor(255, 255, 255, 255)
	love.graphics.draw(atlas, title, (config.display.width / 2) - ( (titleWidth * drawSize * 0.1) / 2), self.title.y, 0, drawSize * 0.1, drawSize * 0.1)

	--distance
	love.graphics.draw(atlas, distance, self.hud.x, config.display.height * 0.005, 0, drawSize / assetSize, drawSize / assetSize)
	--Heart
	for i=1, self.lives do
		love.graphics.draw(atlas, heart, (self.hud.x + (config.display.width * 0.005)) + ( (drawSize * 0.8) * (i - 1)) , config.display.height * 0.1, 0, drawSize / assetSize, drawSize / assetSize)
	end
end

function game:draw()

	--Entity
	love.graphics.setCanvas(self.canvas.entity)
	love.graphics.clear()
	entity:draw()
	--UI
	love.graphics.setCanvas(self.canvas.gui)
	love.graphics.clear()
	self:drawUI()
	love.graphics.setCanvas()

	if self.screenShake.tick > 0 then
		love.graphics.push()
		love.graphics.translate(math.random(-self.screenShake.magnitude, self.screenShake.magnitude), math.random(-self.screenShake.magnitude, self.screenShake.magnitude))
	end

	if self.trip then
		love.graphics.setCanvas(self.canvas.shader)
		
		color:draw(function()
		love.graphics.setColor(255, 255, 255, 255)
		love.graphics.draw(self.sky)

		--Ground
		love.graphics.setColor(self.ground.color)
		love.graphics.rectangle("fill", 0, self.ground.y, config.display.width, config.display.height - self.ground.y)

		love.graphics.setBlendMode("alpha", "premultiplied")
		love.graphics.setColor(255, 255, 255, 255)
		love.graphics.draw(self.canvas.entity)
		end)
		love.graphics.setCanvas()

		wave:draw(function()
		love.graphics.setColor(255, 255, 255, 255)
		love.graphics.draw(self.canvas.shader)
		end)

	else
		love.graphics.setColor(255, 255, 255, 255)
		love.graphics.draw(self.sky)

		--Ground
		love.graphics.setColor(self.ground.color)
		love.graphics.rectangle("fill", 0, self.ground.y, config.display.width, config.display.height - self.ground.y)

		love.graphics.setBlendMode("alpha", "premultiplied")
		love.graphics.setColor(255, 255, 255, 255)
		love.graphics.draw(self.canvas.entity)
	end

	if self.screenShake.tick > 0 then
		love.graphics.pop()
	end

	--Vignette
	love.graphics.setColor(255, 255, 255, 100)
	love.graphics.draw(vignette, 0, 0, 0, config.display.width / vignette:getWidth(), config.display.height / vignette:getHeight())
	
	--Shadow
	love.graphics.setColor(0, 0, 0, 30)
	love.graphics.draw(self.canvas.gui, math.floor( -(config.display.width * 0.003)), 0)

	love.graphics.setColor(255, 255, 255, 255)
	love.graphics.draw(self.canvas.gui)
	love.graphics.setBlendMode("alpha")

	--Debug stuff
	if debugMode then
		love.graphics.setColor(255, 0, 0, 255)
		--[[
		local item = physics.world:queryRect(0, 0, config.display.width, config.display.height)
		for i,v in ipairs(item) do
			love.graphics.rectangle("line", v.x, v.y, v.width, v.height)
		end
		]]
		love.graphics.setFont(font.tiny)
		love.graphics.print(love.timer.getFPS(), 12, 12)
	end
	--love.graphics.print(#entity.array)
end


function game:resize()
	self:load()
end

function game:keypressed(key)
	if self.takeInput then
		if key == config.controls.jump then
			if not self.started then
				self:start()
			elseif self.paused then
				self:resume()
			elseif self.ended then
				self:reset()
			else
				self.player:jump()
			end
		end

		if key == config.controls.pause then
			self:pause()
		end
	end
end

function game:touchpressed(id, x, y, dx, dy, pressure)
	if platform == "mobile" then
		self:input(x, y, "press")
	end
end

function game:touchreleased(id, x, y, dx, dy, pressure)
	if platform == "mobile" then
		self:input(x, y, "release")
	end
end

function game:mousepressed(x, y, key)
	if platform == "pc" then
		self:input(x, y, "mouse")
	end

	--popup:new(heart, config.display.width / 2, config.display.height / 2)
end

function game:input(x, y, t)
	if self.takeInput then
		if y < self.ground.y then
			if t == "press" or t == "mouse" then
				if self.started and not self.paused and not self.ended then
					self.player:jump()
				else
					for i,v in pairs(self.textObject) do
						v:click(x, y, key)
					end
				end
			end
		else
			if t == "release" or t == "mouse" then
				for i,v in pairs(self.textObject) do
					v:click(x, y, key)
				end
			end
		end
	end
end

function game.textClick(t)
	if t.tag == "start" then
		game:start()
	elseif t.tag == "pause" then
		game:pause()
	elseif t.tag == "resume" then
		game:resume()
	elseif t.tag == "reset" then
		game:reset()
	elseif t.tag == "credits" then
		game:showCredits()
	elseif t.tag == "back" then
		game:hideCredits()
	elseif t.tag == "sound" then
		game:setVolume(true)
	elseif t.tag == "love" then
		openURL("https://love2d.org")
	end
end

return game