function love.load()
	--LÖVE Setup
	love.filesystem.setIdentity("Cactus Game")
	love.graphics.setDefaultFilter("nearest", "nearest")

	if not love.filesystem.exists("screenshot") then
		love.filesystem.createDirectory("screenshot")
	end

	local os = love.system.getOS()
	platform = "pc"
	if os == "Android" or os == "iOS"then
		platform = "mobile"
	end
	--platform = "mobile"

	useShader = false

	--Loading Classes
	ini = require "data.class.ini"
	fmath = require "data.class.fmath"
	button = require "data.class.button"
	entity = require "data.class.entity"
	bump = require "data.class.bump"
	physics = require "data.class.physics"
	text = require "data.class.text"
	popup = require "data.class.popup"
	sound = require "data.class.sound"
	console = require "data.class.console"
	shader = require "data.class.shader"
	button = require "data.class.button"

	--loading States
	game = require "data.state.game"

	--Setting up / Loading config file
	local w, h = 1280, 720
	if platform == "mobile" then
		w, h = love.window.getDesktopDimensions()
	end
	config = {
		display = {
			width = w,
			height = h,
			fullscreen = false,
			display = 1,
			windowTitle = "Cactus Game"
		},
		controls = {
			jump = "space",
			pause = "p"
		},
		game = {
			topDistance = 0,
			totalDistance = 0
		},
		sound = {
			volume = 1,
		}
	}

	if love.filesystem.exists("config.ini") then
		config = ini.load("config.ini")
	else
		ini.save("config.ini", config)
	end

	--Creating Window
	love.window.setMode(config.display.width, config.display.height, {resizable = true, fullscreen = config.display.fullscreen, display = config.display.display})
	love.window.setTitle(config.display.windowTitle)

	--Loading Assets
	assetSize = 16
	drawSize = math.floor(config.display.height * 0.1)
	vignette = love.graphics.newImage("data/art/img/vignette.png")
	--title = love.graphics.newImage("data/art/img/title.png")
	loadFont()
	atlas = love.graphics.newImage("data/art/img/image.png")

	--UI Images
	titleWidth = 101
	titleHeight = 33
	title = love.graphics.newQuad(0, 68, titleWidth, titleHeight, atlas:getWidth(), atlas:getHeight())
	loveLogo = love.graphics.newQuad(102, 51, 32, 32, atlas:getWidth(), atlas:getHeight())
	heart = love.graphics.newQuad(0, 51, assetSize, assetSize, atlas:getWidth(), atlas:getHeight())
	distance = love.graphics.newQuad(17, 51, assetSize, assetSize, atlas:getWidth(), atlas:getHeight())

	soundButton1 = love.graphics.newQuad(34, 51, assetSize, assetSize, atlas:getWidth(), atlas:getHeight())
	soundButton2 = love.graphics.newQuad(51, 51, assetSize, assetSize, atlas:getWidth(), atlas:getHeight())
	soundButton3 = love.graphics.newQuad(68, 51, assetSize, assetSize, atlas:getWidth(), atlas:getHeight())
	soundButton4 = love.graphics.newQuad(85, 51, assetSize, assetSize, atlas:getWidth(), atlas:getHeight())


	--Sound fx
	sound:new("data/art/snd/jump.ogg", "jump")
	sound:new("data/art/snd/jumpTrip.ogg", "jumpTrip")
	sound:new("data/art/snd/hit.ogg", "hit")
	sound:new("data/art/snd/ui1.ogg", "ui1")
	sound:new("data/art/snd/ui1.ogg", "ui2")
	sound:new("data/art/snd/run.ogg", "run")
	sound:new("data/art/snd/health.ogg", "life")
	sound:new("data/art/snd/music/game.ogg", "music")
	sound:new("data/art/snd/trip.ogg", "trip")


	sound:setLoop("run", true)
	sound:setLoop("music", true)

	--Defining Entities
	entity:new(require("data.entity.player.player"), "player")
	entity:new(require("data.entity.cactus.cactus"), "cactus")
	entity:new(require("data.entity.mutantCactus.mutantCactus"), "mutantCactus")
	entity:new(require("data.entity.imposterCactus.imposterCactus"), "imposterCactus")
	entity:new(require("data.entity.funnyCactus.funnyCactus"), "funnyCactus")

	--Defining Shaders
	wave = shader.new([[
	extern number strength;
	extern vec2 screen;
	extern highp float time;
	vec4 effect(vec4 color, Image tex, vec2 tc, vec2 sc)
	{
		tc.y = tc.y + sin(  (tc.x * 30.0) + (time * 10.0) ) * (screen.y * 0.0005) * strength;
		return Texel(tex, tc) * color;
	}

	]])

	wave:send("screen", {config.display.width * 0.1, config.display.height * 0.1})
	wave:send("strength", 1)

	color = shader.new([[
	extern number time;
	extern number strength;
	vec4 effect(vec4 color, Image tex, vec2 tc, vec2 sc)
	{
		vec4 pixel = Texel(tex, tc);

		pixel.b = pixel.b + ( (sin(time * 2.0) * 0.5) * strength);
		pixel.g = pixel.g + ( (cos(time * 2.0) * 0.5) * strength);
		return pixel;
	}

	]])

	game:load()

	console:init(0, 0, config.display.width, config.display.height, true, font.tiny)
	console:setVisible(false)

	debugMode = false

end

function loadFont()
	--love.graphics.setLineWidth(math.floor(config.display.width * 0.004))
	love.graphics.setLineWidth(1)
	font = {
		tiny = love.graphics.newFont("data/art/font/PixelOperator8.ttf", math.floor(config.display.width * 0.019)),
		small = love.graphics.newFont("data/art/font/PixelOperator8.ttf", math.floor(config.display.width * 0.03)),
		large = love.graphics.newFont("data/art/font/PixelOperator8.ttf", math.floor(config.display.width * 0.04))
	}
end

function love.update(dt)
	if console:getVisible() then
		console:update(dt)
	else	
		game:update(dt)
	end
end

function love.draw()
	game:draw()
	console:draw()
end

function love.resize(w, h)
	config.display.width = w
	config.display.height = h
	drawSize = math.floor(config.display.height * 0.1) --0.15
	loadFont()
	game:resize()
	console:resize(w, h)
	wave:send("screen", {config.display.width * 0.1, config.display.height * 0.1})
	wave:resize()
	color:resize()
end

function love.textinput(t)
	console:textinput(t)
end

function love.quit()
	ini.save("config.ini", config)
end

function love.keypressed(key)
	if key == "escape" then love.event.push("quit") end

	if console:getVisible() then
		console:keypressed(key)
	else
		game:keypressed(key)
	end

	if key == "f1" then
		debugMode = not debugMode
	elseif key == "f2" then
		screenshot()
	elseif key == "f3" then
		console:setVisible(not console:getVisible())
	end
end

function love.mousepressed(x, y, key)
	game:mousepressed(x, y, key)
end

function love.wheelmoved(x, y)
	if console:getVisible() then
		console:wheelmoved(x, y)
	end
end

function love.touchpressed(id, x, y, dx, dy, pressure)
	game:touchpressed(id, x, y, dx, dy, pressure)
end

function love.touchreleased(id, x, y, dx, dy, pressure)
	game:touchreleased(id, x, y, dx, dy, pressure)
end

function verticalGradient(width, height, ...)
	local canvas = love.graphics.newCanvas(width, height)
	local vertices = {}
	for i,v in ipairs({...}) do
		local y = (height  / (#{...} - 1)) * (i - 1)
		vertices[#vertices + 1] = {
			0, y,
			0, 0,
			v[1], v[2], v[3]
			}
		vertices[#vertices + 1] = {
			width, y,
			0, 0,
			v[1], v[2], v[3]
			}
	end
	local mesh = love.graphics.newMesh(vertices, "strip", "dynamic")
	love.graphics.setCanvas(canvas)
	love.graphics.setColor(255, 255, 255, 255)
	love.graphics.draw(mesh, 0, 0)
	love.graphics.setCanvas()
	return canvas
end

function screenshot()
	local s = love.graphics.newScreenshot()
	local n = os.date("%x")
	s:encode("png", "screenshot/"..os.time()..".png")
end

function openURL(url)
	love.system.openURL(url)
end